﻿using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Web.Mvc;
using MvcSampleApp.Models;
using System;

namespace MvcSampleApp.Controllers
{
    public class BlogsController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Get Json data for the grid
        /// </summary>
        /// <param name="sidx">Column to osrt on.  Do not change parameter name</param>
        /// <param name="sord">Sort direction.  Do not change parameter name</param>
        /// <param name="page">Current page.  Do not change parameter name</param>
        /// <param name="rows">Number of rows to get.  Do not change parameter name</param>
        /// <returns></returns>
        public ActionResult BlogsGridData(string sidx, string sord, int? page, int? rows)
        {
            using (BlogContext context = new BlogContext())
            {
                int pageSize = rows ?? 2;
                int totalRecords = context.Entries.Count();
                int totalPages = (int)Math.Ceiling((float)totalRecords / (float)pageSize);
                int currentPage = page ?? 1;
                string sortByColumnName = sidx ?? "PostedOn";
                string sortDirection = sord ?? "desc";

                var blogs = (from one in context.Entries
                             select one);
                if (sortByColumnName == "PostedOn")
                {
                    if (sortDirection.Equals("desc"))
                        blogs = blogs.OrderByDescending(blog => blog.PostedOn);
                    else
                        blogs = blogs.OrderBy(blog => blog.PostedOn);
                }
                else
                {
                    if ("desc".Equals(sortDirection))
                        blogs = blogs.OrderByDescending(blog => blog.Title);
                    else
                        blogs = blogs.OrderBy(blog => blog.Title);
                }
                var data = blogs.Skip((currentPage - 1) * pageSize).Take(pageSize).ToArray();

                var jsonData = new jqGridJson()
                {
                    total = totalPages,
                    page = currentPage,
                    records = totalRecords,
                    rows = (
                      from blog in data
                      select new jqGridRowJson
                      {
                          id = blog.BlogEntryId.ToString(),
                          cell = new string[] { 
                                "<a href='" + string.Format("./Blogs/Edit/{0}", blog.BlogEntryId.ToString()) + "'>Edit</a>",
                                "<a href='" + string.Format("./Blogs/Delete/{0}", blog.BlogEntryId.ToString()) + "'>Delete</a>",
                                blog.Title, 
                                blog.PostedOn.Value.ToShortDateString() }
                      }).ToArray()
                };

                return Json(jsonData, JsonRequestBehavior.AllowGet);
            }
        }

        public ActionResult Create()
        {
            IEnumerable<BlogCategory> categories = GetCaregories();
            var view = View(new BlogEntry() { Categories = categories });
            view.TempData.Add("Action", "Create");
            return view;
        }

        private static IEnumerable<BlogCategory> GetCaregories()
        {
            IEnumerable<BlogCategory> categories;
            using (BlogContext context = new BlogContext())
            {
                categories = (from one in context.Categories
                              orderby one.CategoryName
                              select one).ToList();
            }
            return categories;
        }

        private BlogCategory GetCategory(int categoryID)
        {
            using (BlogContext context = new BlogContext())
            {
                return context.Categories.Find(categoryID);
            }
        }

        [HttpPost]
        public ActionResult Create(BlogEntry entry)
        {

            try
            {
                if (entry.CategoryID > 0)
                {
                    entry.Category = GetCategory(entry.CategoryID);
                }
                if (TryValidateModel(entry))
                {
                    using (BlogContext context = new BlogContext())
                    {
                        context.Entries.Add(entry);
                        context.SetAsUnchanged(entry.Category);
                        context.SaveChanges();
                    }

                    return RedirectToAction("Index");
                }
                else
                {
                    entry.Categories = GetCaregories();
                    var view = View(entry);
                    view.TempData.Add("Action", "Create");
                    return view;
                }
            }
            catch
            {
                return View();

            }
        }

        //
        // GET: /Blogs/Edit/5

        public ActionResult Edit(int id)
        {
            BlogEntry entry = null;
            using (BlogContext context = new BlogContext())
            {
                entry = context.Entries.Find(id);
                entry.CategoryID = entry.Category.CategoryId;
                entry.Categories = GetCaregories();
            }
            var view = View(entry);
            view.TempData.Add("Action", "Update");
            return view;
        }

        //
        // POST: /Blogs/Edit/5

        [HttpPost]
        public ActionResult Edit(int id, BlogEntry entry)
        {
            try
            {
                if (entry.CategoryID > 0)
                {
                    entry.Category = GetCategory(entry.CategoryID);
                }
                if (TryValidateModel(entry))
                {
                    entry.BlogEntryId = id;
                    using (BlogContext context = new BlogContext())
                    {
                        context.Entries.Attach(entry);
                        context.SetAsModified(entry);
                        context.SaveChanges();
                    }

                    return RedirectToAction("Index");
                }
                else
                {
                    entry.Categories = GetCaregories();
                    var view = View(entry);
                    view.TempData.Add("Action", "Update");
                    return view;
                }
            }
            catch
            {

                var view = View();
                view.TempData.Add("Action", "Update");
                return view;
            }
        }

        //
        // GET: /Blogs/Delete/5

        public ActionResult Delete(int id)
        {
            BlogEntry entry = null;
            using (BlogContext context = new BlogContext())
            {
                entry = context.Entries.Find(id);
                entry.CategoryID = entry.Category.CategoryId;
                entry.Categories = GetCaregories();
            }
            var view = View(entry);
            view.TempData.Add("Action", "Delete");
            return view;
        }

        //
        // POST: /Blogs/Delete/5

        [HttpPost]
        public ActionResult Delete(int id, BlogEntry entry)
        {
            try
            {
                if (entry.CategoryID > 0)
                {
                    entry.Category = GetCategory(entry.CategoryID);
                }
                if (TryValidateModel(entry))
                {
                    entry.BlogEntryId = id;
                    using (BlogContext context = new BlogContext())
                    {
                        context.Entries.Attach(entry);
                        context.Entries.Remove(entry);
                        context.SaveChanges();
                    }

                    return RedirectToAction("Index");
                }
                else
                {
                    entry.Categories = GetCaregories();
                    var view = View(entry);
                    view.TempData.Add("Action", "Delete");
                    return view;
                }
            }
            catch
            {
                return View();

            }
        }
    }
}
